import { reactive, toRaw } from "vue";

// 分页钩子函数
interface Options {
  page?: number;
  size?: number;
  fetchFun: (_arg: any) => Promise<any>;
  params?: Record<any, any>;
  firstLoading?: boolean;
}

interface Icolumn {
  column: { label: String; width: Number; property: string };
}

/**
 * @description 计算table中每一项渲染到dom上所需要的宽度
 * @param string:需要渲染到dom上的内容
 * */
function getWidthOfString(string: String) {
  var span = document.createElement("span");
  // @ts-ignore
  span.textContent = string;
  span.style.visibility = "hidden";
  span.style.position = "absolute";
  span.style.whiteSpace = "nowrap";
  document.body.appendChild(span);
  var width = span.offsetWidth;
  document.body.removeChild(span);
  return width;
}

export function usePaging(options: Options) {
  const {
    page = 1,
    size = 15,
    fetchFun,
    params = {},
    firstLoading = false,
  } = options;
  // 记录分页初始参数
  const paramsInit: Record<any, any> = Object.assign({}, toRaw(params));
  // 分页数据
  const pager = reactive({
    page,
    size,
    loading: firstLoading,
    count: 0,
    lists: [] as any[],
    extend: {} as Record<string, any>,
    calcWidth: ({ column }: Icolumn) => {
      // if (!pager.lists?.length) return column.label;

      let propWidth = column.label.length * 14 + 30;
      let list = pager.lists.map((item: Object) =>
        column.property?.includes(
          "."
        ) /**用于el-columnc传入深层次的对象属性，如projectInfo.contract */
          ? (item as any)[column.property.split(".")[0]][
              column.property.split(".")[1]
            ]
          : (item as any)[column.property]
      );
      // start 遍历获取lits每一项所需要的宽度的最大值
      const maxLength = list.reduce((max, str) => {
        return Math.max(max, getWidthOfString(str));
      }, 0);
      // end

      const columnWidth = maxLength + 27; //27为el-table的cell 自带的padding 12px*2 + border 1px * 2 + 预留2px防止妖魔鬼怪作乱
      column.width = Math.max(columnWidth, propWidth);
      return column.label;
    },
  });
  // 请求分页接口
  const getLists = () => {
    pager.loading = true;
    return fetchFun({
      page_no: pager.page,
      page_size: pager.size,
      ...params,
    })
      .then((res: any) => {
        pager.count = res?.count;
        pager.lists = res?.lists;
        pager.extend = res?.extend;
        return Promise.resolve(res);
      })
      .catch((err: any) => {
        return Promise.reject(err);
      })
      .finally(() => {
        pager.loading = false;
      });
  };
  // 重置为第一页
  const resetPage = () => {
    pager.page = 1;
    getLists();
  };
  // 重置参数
  const resetParams = () => {
    Object.keys(paramsInit).forEach((item) => {
      params[item] = paramsInit[item];
    });
    getLists();
  };
  return {
    pager,
    getLists,
    resetParams,
    resetPage,
  };
}
